@@ -2,13 +2,17 @@
 
 ;;; Code:
 
+(autoload 'custom-add-loads "cus-load")
+
 (custom-add-loads 'filladapt '("filladapt"))
 (custom-add-loads 'xrdb '("xrdb-mode"))
 (custom-add-loads 'tools '("autoinsert" "folding"))
+(custom-add-loads 'hypermedia '("htmlize"))
 (custom-add-loads 'outlines '("whitespace-mode"))
 (custom-add-loads 'crontab '("crontab"))
 (custom-add-loads 'scribe '("scribe"))
 (custom-add-loads 'auto-insert '("autoinsert"))
+(custom-add-loads 'htmlize '("htmlize"))
 (custom-add-loads 'flyspell '("flyspell"))
 (custom-add-loads 'languages '("crontab" "winmgr-mode" "xrdb-mode"))
 (custom-add-loads 'fill '("filladapt"))
